-- Abstract watchdog timer device interface definitions

-- Copyright (C)2017-2023, Philip Munts dba Munts Technologies.
--
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- * Redistributions of source code must retain the above copyright notice,
--   this list of conditions and the following disclaimer.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.

PACKAGE Watchdog IS

  -- Define an exception for watchdog timer device errors

  Watchdog_Error : EXCEPTION;

  -- Define an abstract interface for watchdog timer devices

  TYPE TimerInterface IS INTERFACE;

  -- Define an access type compatible with any subclass implementing
  -- Watchdog.TimerInterface

  TYPE Timer IS ACCESS ALL Watchdog.TimerInterface'Class;

  -- Method to get the watchdog timeout in seconds

  FUNCTION GetTimeout(Self : TimerInterface) RETURN Duration IS ABSTRACT;

  -- Method to change the watchdog timeout in seconds

  PROCEDURE SetTimeout(Self : TimerInterface; timeout : Duration) IS ABSTRACT;

  -- Method to reset the watchdog timer

  PROCEDURE Kick(Self : TimerInterface) IS ABSTRACT;

END Watchdog;
