-- Generic device definitions

-- Copyright (C)2018-2023, Philip Munts dba Munts Technologies.
--
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- * Redistributions of source code must retain the above copyright notice,
--   this list of conditions and the following disclaimer.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.

WITH Ada.Strings.Fixed;
WITH Ada.Text_IO;

PACKAGE BODY Device IS

  PACKAGE NaturalIO IS NEW Ada.Text_IO.Integer_IO(Natural);

  -- Get device designator chip and channel from operator

  FUNCTION GetDesignator(prompt : String) RETURN Designator IS

    i    : CONSTANT Natural := Ada.Strings.Fixed.Index(prompt, ": ");
    desg : Designator       := Unavailable;

  BEGIN
    IF i = 0 THEN
      Ada.Text_IO.Put(prompt & " chip:    ");
    ELSE
      Ada.Text_IO.Put(Ada.Strings.Fixed.Overwrite(prompt, i, " chip:    "));
    END IF;

    NaturalIO.Get(desg.chip);

    IF i = 0 THEN
      Ada.Text_IO.Put(prompt & " channel: ");
    ELSE
      Ada.Text_IO.Put(Ada.Strings.Fixed.Overwrite(prompt, i, " channel: "));
    END IF;

    NaturalIO.Get(desg.chan);
    RETURN Desg;
  END GetDesignator;

  -- Get device designator channel from operator

  FUNCTION GetDesignator(chip : Natural; prompt : String) RETURN Designator IS

    i    : CONSTANT Natural := Ada.Strings.Fixed.Index(prompt, ": ");
    desg : Designator       := Unavailable;

  BEGIN
    desg.chip := chip;

    IF i = 0 THEN
      Ada.Text_IO.Put(prompt & " channel: ");
    ELSE
      Ada.Text_IO.Put(Ada.Strings.Fixed.Overwrite(prompt, i, " channel: "));
    END IF;

    NaturalIO.Get(desg.chan);
    RETURN Desg;
  END GetDesignator;

END Device;
